package com.hz.pm.api.projectlib.handle;

import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.hz.pm.api.projectlib.model.enumeration.TenderAdaptStatusEnum;
import com.hz.pm.api.todocenter.utils.BuildUserUtils;
import com.wflow.workflow.bean.process.ProgressNode;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

/**
 * <p>
 * AdaptionHandle
 * </p>
 *
 * @author WendyYang
 * @since 09:29 2024/4/9
 */
@Component
@RequiredArgsConstructor
public class ProjectAdaptionHandle extends AbstractProcessHandle {

    private final IPurchaseInstService purchaseInstService;
    private final BuildUserUtils buildUserUtils;
    private final ProcessInstanceService processInstanceService;

    @Override
    void handle(HandleParamContext ctx) {
        if (!ctx.getPreStepStatus().equals(StepStatusEnum.COMPLETED)) {
            return;
        }
        List<Purchase> purchases = ctx.getPurchases();
        Map<Long, Purchase> purchaseMap = purchases.stream()
                .filter(w -> TenderAdaptStatusEnum.ADAPT_INFO_PASSED.eq(w.getAdaptStatus())
                        || TenderAdaptStatusEnum.ADAPT_INFO_FAILED.eq(w.getAdaptStatus())
                        || TenderAdaptStatusEnum.ADAPT_INFO_AUDIT.eq(w.getAdaptStatus()))
                .collect(Collectors.toMap(Purchase::getId, Function.identity()));
        if (purchaseMap.isEmpty()) {
            return;
        }
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        detail.setProcessInfoList(new ArrayList<>());
        ctx.getProcessDetails().add(detail);
        Map<Long, PurchaseInst> instMap = purchaseInstService.list(purchaseMap.keySet(), InstTypeEnum.ADAPTION.getCode());
        List<ProgressNode> progressNodes = new ArrayList<>();
        instMap.forEach((bidId, inst) -> {
            if (detail.getFinishTime() == null || detail.getFinishTime().isBefore(inst.getCreateOn())) {
                detail.setFinishTime(inst.getCreateOn());
            }
            ProcessProgressVo instanceDetail = processInstanceService.getProgressInstanceDetail(null, inst.getInstCode());
            progressNodes.addAll(instanceDetail.getProgressInfo());
            ProcessInfoVO tenderProgress = new ProcessInfoVO();
            tenderProgress.setProcessName(purchaseMap.get(bidId).getBidName());
            tenderProgress.setProcessProgressVo(instanceDetail);
            detail.getProcessInfoList().add(tenderProgress);
        });
        buildUserUtils.buildUserByProcessInfo(progressNodes);
        StepStatusEnum stepStatus = getPurchaseStatus(detail.getProcessInfoList());
        detail.setStepStatus(stepStatus);
        ctx.setPreStepStatus(stepStatus);
    }

    @Override
    ProcessNode getProcessNode() {
        return ProcessNode.PROJECT_ADAPTION;
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

}
