package com.hz.pm.api.projectlib.handle;

import com.hz.pm.api.projectdeclared.model.entity.Purchase;
import com.hz.pm.api.projectdeclared.model.entity.PurchaseInst;
import com.hz.pm.api.projectdeclared.service.IPurchaseInstService;
import com.hz.pm.api.projectdeclared.service.IPurchaseService;
import com.hz.pm.api.projectlib.model.enumeration.InstTypeEnum;
import com.hz.pm.api.todocenter.utils.BuildUserUtils;
import com.wflow.workflow.bean.process.ProgressNode;
import com.wflow.workflow.bean.vo.ProcessDetailVO;
import com.wflow.workflow.bean.vo.ProcessInfoVO;
import com.wflow.workflow.bean.vo.ProcessProgressVo;
import com.wflow.workflow.enums.StepStatusEnum;
import com.wflow.workflow.service.ProcessInstanceService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import static com.hz.pm.api.projectlib.model.enumeration.TenderSelfTestStatusEnum.*;

/**
 * <p>
 * SelfTestHandle
 * </p>
 *
 * @author WendyYang
 * @since 09:29 2024/4/9
 */
@Component
@RequiredArgsConstructor
public class ProjectSelfTestHandle extends AbstractProcessHandle {

    private final IPurchaseService purchaseService;
    private final IPurchaseInstService purchaseInstService;
    private final BuildUserUtils buildUserUtils;
    private final ProcessInstanceService processInstanceService;

    @Override
    void handle(HandleParamContext ctx) {
        if (!ctx.getPreStepStatus().equals(StepStatusEnum.COMPLETED)) {
            return;
        }
        List<Purchase> purchases = ctx.getPurchases();
        Map<Long, Purchase> purchaseMap = purchases.stream()
                .filter(w -> SELF_TEST_INFO_PASSED.eq(w.getSelfTestStatus())
                        || SELF_TEST_INFO_FAILED.eq(w.getSelfTestStatus())
                        || SELF_TEST_INFO_AUDIT.eq(w.getSelfTestStatus()))
                .collect(Collectors.toMap(Purchase::getId, Function.identity()));
        if (purchaseMap.isEmpty()) {
            return;
        }
        ProcessDetailVO detail = new ProcessDetailVO();
        detail.setProcessName(getProcessNode().getName());
        detail.setProcessInfoList(new ArrayList<>());
        ctx.getProcessDetails().add(detail);
        Map<Long, PurchaseInst> instMap = purchaseInstService.list(purchaseMap.keySet(), InstTypeEnum.SELF_TEST.getCode());
        List<ProgressNode> progressNodes = new ArrayList<>();
        for (Map.Entry<Long, PurchaseInst> entry : instMap.entrySet()) {
            if (detail.getFinishTime() == null || detail.getFinishTime().isBefore(entry.getValue().getCreateOn())) {
                detail.setFinishTime(entry.getValue().getCreateOn());
            }
            ProcessProgressVo instanceDetail = processInstanceService.getProgressInstanceDetail(null, entry.getValue().getInstCode());
            progressNodes.addAll(instanceDetail.getProgressInfo());
            ProcessInfoVO tenderProgress = new ProcessInfoVO();
            tenderProgress.setProcessName(purchaseMap.get(entry.getKey()).getBidName());
            tenderProgress.setProcessProgressVo(instanceDetail);
            detail.getProcessInfoList().add(tenderProgress);
        }
        buildUserUtils.buildUserByProcessInfo(progressNodes);
        StepStatusEnum stepStatus = getPurchaseStatus(detail.getProcessInfoList());
        detail.setStepStatus(stepStatus);
        ctx.setPreStepStatus(stepStatus);
    }

    @Override
    ProcessNode getProcessNode() {
        return ProcessNode.SYSTEM_SELF_TEST;
    }

    @Override
    public int getOrder() {
        return getProcessNode().getOrder();
    }

}
